@@ -1,4 +1,4 @@
-/*	$OpenBSD: roa.c,v 1.9 2020/09/12 15:46:48 claudio Exp $ */
+/*	$OpenBSD: roa.c,v 1.17 2021/03/29 06:50:44 tb Exp $ */
 /*
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -23,6 +23,9 @@
 #include <string.h>
 #include <unistd.h>
 
+#include <openssl/asn1.h>
+#include <openssl/x509.h>
+
 #include "extern.h"
 
 /*
@@ -108,12 +111,11 @@ roa_parse_addr(const ASN1_OCTET_STRING *os, enum afi afi, struct parse *p)
 		/* FIXME: maximum check. */
 	}
 
-	p->res->ips = reallocarray(p->res->ips,
-		p->res->ipsz + 1, sizeof(struct roa_ip));
+	p->res->ips = recallocarray(p->res->ips, p->res->ipsz, p->res->ipsz + 1,
+	    sizeof(struct roa_ip));
 	if (p->res->ips == NULL)
 		err(1, NULL);
 	res = &p->res->ips[p->res->ipsz++];
-	memset(res, 0, sizeof(struct roa_ip));
 
 	res->addr = addr;
 	res->afi = afi;
@@ -323,13 +325,11 @@ out:
 }
 
 /*
- * Parse a full RFC 6482 file with a SHA256 digest "dgst" and signed by
- * the certificate "cacert" (the latter two are optional and may be
- * passed as NULL to disable).
+ * Parse a full RFC 6482 file.
  * Returns the ROA or NULL if the document was malformed.
  */
 struct roa *
-roa_parse(X509 **x509, const char *fn, const unsigned char *dgst)
+roa_parse(X509 **x509, const char *fn)
 {
 	struct parse	 p;
 	size_t		 cmsz;
@@ -342,14 +342,22 @@ roa_parse(X509 **x509, const char *fn, const unsigned char *dgst)
 	/* OID from section 2, RFC 6482. */
 
 	cms = cms_parse_validate(x509, fn,
-	    "1.2.840.113549.1.9.16.1.24", dgst, &cmsz);
+	    "1.2.840.113549.1.9.16.1.24", &cmsz);
 	if (cms == NULL)
 		return NULL;
 
 	if ((p.res = calloc(1, sizeof(struct roa))) == NULL)
 		err(1, NULL);
-	if (!x509_get_ski_aki(*x509, fn, &p.res->ski, &p.res->aki))
+
+	p.res->aia = x509_get_aia(*x509, fn);
+	p.res->aki = x509_get_aki(*x509, 0, fn);
+	p.res->ski = x509_get_ski(*x509, fn);
+	if (p.res->aia == NULL || p.res->aki == NULL || p.res->ski == NULL) {
+		warnx("%s: RFC 6487 section 4.8: "
+		    "missing AIA, AKI or SKI X509 extension", fn);
 		goto out;
+	}
+
 	if (!roa_parse_econtent(cms, cmsz, &p))
 		goto out;
 
@@ -376,6 +384,7 @@ roa_free(struct roa *p)
 
 	if (p == NULL)
 		return;
+	free(p->aia);
 	free(p->aki);
 	free(p->ski);
 	free(p->ips);
@@ -388,29 +397,26 @@ roa_free(struct roa *p)
  * See roa_read() for reader.
  */
 void
-roa_buffer(char **b, size_t *bsz, size_t *bmax, const struct roa *p)
+roa_buffer(struct ibuf *b, const struct roa *p)
 {
 	size_t	 i;
 
-	io_simple_buffer(b, bsz, bmax, &p->valid, sizeof(int));
-	io_simple_buffer(b, bsz, bmax, &p->asid, sizeof(uint32_t));
-	io_simple_buffer(b, bsz, bmax, &p->ipsz, sizeof(size_t));
+	io_simple_buffer(b, &p->valid, sizeof(int));
+	io_simple_buffer(b, &p->asid, sizeof(uint32_t));
+	io_simple_buffer(b, &p->ipsz, sizeof(size_t));
 
 	for (i = 0; i < p->ipsz; i++) {
-		io_simple_buffer(b, bsz, bmax,
-		    &p->ips[i].afi, sizeof(enum afi));
-		io_simple_buffer(b, bsz, bmax,
-		    &p->ips[i].maxlength, sizeof(size_t));
-		io_simple_buffer(b, bsz, bmax,
-		    p->ips[i].min, sizeof(p->ips[i].min));
-		io_simple_buffer(b, bsz, bmax,
-		    p->ips[i].max, sizeof(p->ips[i].max));
-		ip_addr_buffer(b, bsz, bmax, &p->ips[i].addr);
+		io_simple_buffer(b, &p->ips[i].afi, sizeof(enum afi));
+		io_simple_buffer(b, &p->ips[i].maxlength, sizeof(size_t));
+		io_simple_buffer(b, p->ips[i].min, sizeof(p->ips[i].min));
+		io_simple_buffer(b, p->ips[i].max, sizeof(p->ips[i].max));
+		ip_addr_buffer(b, &p->ips[i].addr);
 	}
 
-	io_str_buffer(b, bsz, bmax, p->aki);
-	io_str_buffer(b, bsz, bmax, p->ski);
-	io_str_buffer(b, bsz, bmax, p->tal);
+	io_str_buffer(b, p->aia);
+	io_str_buffer(b, p->aki);
+	io_str_buffer(b, p->ski);
+	io_str_buffer(b, p->tal);
 }
 
 /*
@@ -442,9 +448,12 @@ roa_read(int fd)
 		ip_addr_read(fd, &p->ips[i].addr);
 	}
 
+	io_str_read(fd, &p->aia);
 	io_str_read(fd, &p->aki);
 	io_str_read(fd, &p->ski);
 	io_str_read(fd, &p->tal);
+	assert(p->aia && p->aki && p->ski && p->tal);
+
 	return p;
 }
 
